# -*- coding: utf-8 -*-
# Copyright 2017-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import properties

from .. import BaseModel


class Email(BaseModel):

    value = properties.String(
        'Email Address',
        required=True,
    )
    location = properties.StringChoice(
        'Location for this email address.',
        choices=['home', 'work', 'other'],
        default='other',
        required=True,
    )



# -*- coding: utf-8 -*-
# Copyright 2017-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import properties


class BaseModel(properties.HasProperties):
    """This is the model that all other models inherit from.

    It provides some convenience functions, and the standard ``id`` property.
    """

    id = properties.Integer(
        'Unique identifier',
    )



# -*- coding: utf-8 -*-
# Copyright 2017-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import properties

from .. import BaseModel


class Website(BaseModel):

    value = properties.String(
        'URI',
        required=True,
    )



# -*- coding: utf-8 -*-
# Copyright 2017-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from .. import BaseApi

from ..models.conversation import Conversation
from ..models.thread import Thread

from ..request_paginator import RequestPaginator


class Conversations(BaseApi):
    """This represents the ``Conversations`` Endpoint.

    The following aspects are implemented:

    * `List Conversations
      <developer.helpscout.net/help-desk-api/conversations/list/>`_
    * `Get Conversation
      <developer.helpscout.net/help-desk-api/conversations/get/>`_
    * `Create Conversation
      <developer.helpscout.net/help-desk-api/conversations/create/>`_
    * `Update Conversation
      <developer.helpscout.net/help-desk-api/conversations/update/>`_
    * `Delete Conversation
      <developer.helpscout.net/help-desk-api/conversations/delete/>`_
    * `Create Thread
      <developer.helpscout.net/help-desk-api/conversations/create-thread/>`_
    * `Update Thread
      <developer.helpscout.net/help-desk-api/conversations/update-thread/>`_
    """

    __object__ = Conversation

    @classmethod
    def create(cls, session, conversation, imported=False, auto_reply=False,
               _reload=False):
        """Create a conversation.

        Please note that conversation cannot be created with more than 100
        threads, if attempted the API will respond with HTTP 412.

        Args:
            session (requests.Session): Authenticated requests Session.
            conversation (Conversation): The conversation to be created.
            imported (bool): The ``imported`` request parameter enables
            conversations to be created for historical purposes (i.e.
            if moving from a different platform, you can import your
            history). When ``imported`` is set to ``True``, no outgoing emails
            or notifications will be generated.
            auto_reply (bool): The ``auto_reply`` request parameter enables
            auto replies to be sent when a conversation is created via the
            API. When ``auto_reply`` is set to ``True``, an auto reply will be
            sent as long as there is at least one ``customer`` thread in the
            conversation.
            _reload (bool, optional): Set this request parameter to ``True``
            to return the created conversation in the response.

        Returns:
            bool: Success status, if ``_reload`` is ``False``.
            Conversation: Newly created conversation, if ``_reload`` is
            ``True``.
        """
        data = {
            'conversation': conversation.to_api(),
            'imported': imported,
            'auto_reply': auto_reply,
            'reload': _reload,
        }
        return cls(
            '/conversations.json',
            data=data,
            request_type=RequestPaginator.POST,
            singleton=True,
            session=session,
        )

    @classmethod
    def create_thread(cls, session, conversation, thread, imported=False,
                      _reload=False):
        """Create a conversation thread.

        Please note that threads cannot be added to conversations with 100 threads
        (or more), if attempted the API will respond with HTTP 412.

        Args:
            conversation (Conversation): The conversation that the thread is
            being added to.
            session (requests.Session): Authenticated requests Session.
            thread (Thread): The thread to be created.
            imported (bool): The ``imported`` request parameter enables
            threads to be created for historical purposes (i.e.
            if moving from a different platform, you can import your
            history). When ``imported`` is set to ``True``, no outgoing emails
            or notifications will be generated.
            _reload (bool, optional): Set this request parameter to ``True``
            to return the created conversation in the response.

        Returns:
            bool: Success status, if ``_reload`` is ``False``.
            Thread: Newly created thread, if ``_reload`` is ``True``.
        """
        data = {
            'thread': thread.to_api(),
            'imported': imported,
            'reload': _reload,
        }
        return cls(
            '/conversations/%s.json' % conversation.id,
            data=data,
            request_type=RequestPaginator.POST,
            singleton=True,
            session=session,
        )

    @classmethod
    def delete(cls, session, conversation):
        """Delete a conversation.

        Args:
            session (requests.Session): Authenticated requests Session.
            conversation (Conversation): The conversation to be deleted.

        Returns:
            bool: Status
        """
        return cls(
            '/conversations/%s.json' % conversation.id,
            request_type=RequestPaginator.DELETE,
            singleton=True,
            session=session,
        )

    @classmethod
    def get(cls, session, conversation_id):
        """Return a specific conversation.

        Args:
            session (requests.Session): Authenticated requests Session.
            conversation_id (int): The ID of the conversation to get.

        Returns:
            Conversation: A conversation singleton, if existing.
            None: If no match for the ID.
        """
        return cls(
            '/conversations/%d.json' % conversation_id,
            singleton=True,
            session=session,
        )

    @classmethod
    def update(cls, session, conversation, _reload=False):
        """Update a conversation.

        Args:
            session (requests.Session): Authenticated requests Session.
            conversation (Conversation): The conversation to be updated.
            _reload (bool, optional): Set this request parameter to ``True``
            to return the created conversation in the response.

        Returns:
            bool: Success status, if ``_reload`` is ``False``.
            Conversation: Freshly updated conversation, if ``_reload`` is
            ``True``.
        """
        data = {
            'conversation': conversation.to_api(),
            'reload': _reload,
        }
        return cls(
            '/conversations/%s.json' % conversation.id,
            data=data,
            request_type=RequestPaginator.PUT,
            singleton=True,
            session=session,
        )

    @classmethod
    def update_thread(cls, session, conversation, thread, _reload=False):
        """Update a thread.

        Args:
            session (requests.Session): Authenticated requests Session.
            conversation (Conversation): The conversation that the thread
            belongs to.
            thread (Thread): The thread to be updated.
            _reload (bool, optional): Set this request parameter to ``True``
            to return the created conversation in the response.

        Returns:
            bool: Success status, if ``_reload`` is ``False``.
            Thread: Freshly updated thread, if ``_reload`` is ``True``.
        """
        data = {
            'thread': thread.to_api(),
            'reload': _reload,
        }
        return cls(
            '/conversations/%s/threads/%d.json' % (
                conversation.id, thread.id,
            ),
            data=data,
            request_type=RequestPaginator.PUT,
            singleton=True,
            session=session,
        )

    @classmethod
    def list(cls, session, mailbox):
        """Return conversations in a mailbox.

        Args:
            session (requests.Session): Authenticated requests Session.
            mailbox (helpscout.models.Mailbox): Mailbox to list.

        Returns:
            RequestPaginator of Conversation: Conversations iterator.
        """
        return cls(
            '/mailboxes/%d/conversations.json' % mailbox.id,
            session=session,
        )

    @classmethod
    def list_folder(cls, session, mailbox, folder):
        """Return conversations in a specific folder of a mailbox.

        Args:
            session (requests.Session): Authenticated requests Session.
            mailbox (helpscout.models.Mailbox): Mailbox that folder is in.
            folder (helpscout.models.Folder): Folder to list.

        Returns:
            RequestPaginator of Conversation: Conversations iterator.
        """
        return cls(
            '/mailboxes/%d/folders/%s/conversations.json' % (
                mailbox.id, folder.id,
            ),
            session=session,
        )

    @classmethod
    def find_customer(cls, session, mailbox, customer):
        """Return conversations for a specific customer in a mailbox.

        Args:
            session (requests.Session): Authenticated requests Session.
            mailbox (helpscout.models.Mailbox): Mailbox to search.
            customer (helpscout.models.Customer): Customer to search for.

        Returns:
            RequestPaginator of Conversation: Conversations iterator.
        """
        return cls(
            '/mailboxes/%d/customers/%s/conversations.json' % (
                mailbox.id, customer.id,
            ),
            session=session,
        )

    @classmethod
    def find_user(cls, session, mailbox, user):
        """Return conversations for a specific user in a mailbox.

        Args:
            session (requests.Session): Authenticated requests Session.
            mailbox (helpscout.models.Mailbox): Mailbox to search.
            user (helpscout.models.User): User to search for.

        Returns:
            RequestPaginator of Conversation: Conversations iterator.
        """
        return cls(
            '/mailboxes/%d/users/%s/conversations.json' % (
                mailbox.id, user.id,
            ),
            session=session,
        )



# -*- coding: utf-8 -*-
# Copyright 2017-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import properties

from .. import BaseModel

from .option import Option


class CustomField(BaseModel):
    """This represents optional data that can defined for specific mailbox
    and filled when creating or updating a Conversation."""

    field_name = properties.String(
        'The name of the field; note that this may change if a field '
        'is renamed, but the ``id`` will not.',
        required=True,
    )
    field_type = properties.StringChoice(
        'Type of the field.',
        choices=['SINGLE_LINE',
                 'MULTI_LINE',
                 'DATA',
                 'NUMBER',
                 'DROPDOWN',
                 ],
        default='SINGLE_LINE',
        required=True,
    )
    required = properties.Bool(
        'Flag for UI to mark the field as required.',
    )
    order = properties.Integer(
        'Relative order of the custom field. Can be ``null`` or a number '
        'between ``0`` and ``255``.',
        min=0,
        max=255,
    )
    options = properties.List(
        'Field options',
        prop=Option,
    )



# -*- coding: utf-8 -*-
# Copyright 2017-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from .. import BaseApi

from ..models.user import User


class Teams(BaseApi):
    """This represents the ``Teams`` Endpoint.

    The following aspects are implemented:

    * `List Teams
      <http://developer.helpscout.net/help-desk-api/teams/list/>`_
    * `Get Teams
      <http://developer.helpscout.net/help-desk-api/teams/get/>`_
    * `Get Team Members
      <http://developer.helpscout.net/help-desk-api/teams/team-members/>`_
    """

    __object__ = User

    @classmethod
    def get(cls, session, team_id):
        """Return a specific team.

        Args:
            session (requests.Session): Authenticated requests Session.
            team_id (int): The ID of the team to get.

        Returns:
            Team: A team singleton, if existing.
            None: If no match for the ID.
        """
        return cls(
            '/teams/%d.json' % team_id,
            singleton=True,
            session=session,
        )

    @classmethod
    def list(cls, session):
        """List the teams.

        Args:
            session (requests.Session): Authenticated requests Session.

        Returns:
            RequestPaginator of Team: Teams iterator.
        """
        return cls('/teams.json', session=session)

    @classmethod
    def list_members(cls, session, team_or_id):
        """List the members for the team.

        Args:
            team_or_id (Team or int): Team of the ID of the team
            to get the folders for.

        Returns:
            RequestPaginator of User: Users iterator.
        """
        if isinstance(team_or_id, Team):
            team_or_id = team_or_id.id
        return cls(
            '/teams/%d/members.json' % team_or_id,
            session=session,
        )



# -*- coding: utf-8 -*-
# Copyright 2017-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import properties
import unittest

try:
    import mock
except ImportError:
    mock = unittest.mock

from .. import BaseModel
from ..exceptions import HelpScoutValidationError


class TestModel(BaseModel):
    a_key = properties.String('A key')
    sub_instance = properties.Instance(
        'Sub Instance', instance_class=BaseModel,
    )
    list = properties.List(
        'List', prop=properties.Instance('List Instance',
                                         instance_class=BaseModel),
    )


class TestBaseModel(unittest.TestCase):

    def setUp(self):
        super(TestBaseModel, self).setUp()
        self.test_values = {
            'aKey': 'value',
            'subInstance': {
                'id': 1234,
            },
            'list': [
                {'id': 4321},
            ],
        }
        self.internal_values = {
            'a_key': self.test_values['aKey'],
            'sub_instance': self.test_values['subInstance'],
            'list': self.test_values['list'],
        }

    def new_record(self):
        return TestModel.from_api(**self.test_values)

    def test_from_api_new_instance(self):
        """It should return a new instance of the class."""
        self.assertIsInstance(self.new_record(), TestModel)

    def test_from_api_camel_to_snake(self):
        """It should call the init with snake cased keys."""
        self.assertEqual(self.new_record().a_key, self.test_values['aKey'])

    def test_from_api_invalid_attribute(self):
        """It should not allow invalid attribute assignments."""
        with self.assertRaises(HelpScoutValidationError):
            BaseModel.from_api(**{'no exist': 'value'})

    def test_from_api_sub_instance(self):
        """It should properly instantiate sub-instances."""
        self.assertIsInstance(self.new_record().sub_instance, BaseModel)

    def test_from_api_list(self):
        """It should properly convert lists of instances."""
        res = self.new_record()
        self.assertIsInstance(res.list, list)
        self.assertIsInstance(res.list[0], BaseModel)

    def test_to_api_camel_case(self):
        """It should properly camel case the API args."""
        res = self.new_record().to_api()
        self.assertEqual(res['aKey'], self.internal_values['a_key'])

    def test_to_api_instance(self):
        """It should properly convert sub-instances to dict."""
        res = self.new_record().to_api()
        self.assertIsInstance(res['subInstance'], dict)

    def test_to_api_list(self):
        """It should properly convert sub-instances within lists props."""
        res = self.new_record().to_api()
        self.assertIsInstance(res['list'], list)
        self.assertIsInstance(res['list'][0], dict)

    def test_get_non_empty_vals(self):
        """It should return the dict without NoneTypes."""
        expect = {
            'good_int': 1234,
            'good_false': False,
            'good_true': True,
            'bad': None,
        }
        res = BaseModel.get_non_empty_vals(expect)
        del expect['bad']
        self.assertDictEqual(res, expect)



# -*- coding: utf-8 -*-
# Copyright 2017-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from .. import BaseApi

from ..models.user import User


class Users(BaseApi):
    """This represents the ``Users`` Endpoint.

    The following aspects are implemented:
    
    * `List Users
      <http://developer.helpscout.net/help-desk-api/users/list/>`_
    * `Get Users
      <http://developer.helpscout.net/help-desk-api/users/get/>`_
    * `Get Folders
      <http://developer.helpscout.net/help-desk-api/users/folders/>`_
    """

    __object__ = User

    @classmethod
    def get(cls, session, user_id):
        """Return a specific user.

        Args:
            session (requests.Session): Authenticated requests Session.
            user_id (int): The ID of the user to get.

        Returns:
            User: A user singleton, if existing.
            None: If no match for the ID.
        """
        return cls(
            '/users/%d.json' % user_id,
            singleton=True,
            session=session,
        )

    @classmethod
    def get_me(cls, session):
        """Return the current user.

        Args:
            session (requests.Session): Authenticated requests Session.

        Returns:
            User: A user singleton
        """
        return cls(
            '/users/me.json',
            singleton=True,
            session=session,
        )

    @classmethod
    def list(cls, session):
        """List the users.

        Args:
            session (requests.Session): Authenticated requests Session.

        Returns:
            RequestPaginator of User: Users iterator.
        """
        return cls('/users.json', session=session)

    @classmethod
    def find_in_mailbox(cls, session, mailbox_or_id):
        """Get the users that are associated to a Mailbox.

        Args:
            session (requests.Session): Authenticated requests Session.
            mailbox_or_id (MailboxRef or int): Mailbox of the ID of the
            mailbox to get the folders for.

        Returns:
            RequestPaginator of User: Users iterator.
        """
        if isinstance(mailbox_or_id, User):
            mailbox_or_id = mailbox_or_id.id
        return cls(
            '/mailboxes/%d/users.json' % mailbox_or_id,
            session=session,
        )



# -*- coding: utf-8 -*-
# Copyright 2017-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import properties

from .. import BaseModel

from .user import User


class Folder(BaseModel):

    name = properties.String(
        'Folder name',
        required=True,
    )
    type = properties.StringChoice(
        'The type of folder.',
        choices=['unassigned',
                 'mytickets',
                 'needsattention',
                 'drafts',
                 'assigned',
                 'closed',
                 'spam',
                 ],
        default='drafts',
        required=True,
    )
    user_id = properties.Instance(
        'If the folder type is ``MyTickets``, this represents the Help Scout '
        'user to which this folder belongs. Otherwise it is empty.',
        instance_class=User,
    )
    total_count = properties.Integer(
        'Total number of conversations in this folder.',
    )
    active_count = properties.Integer(
        'Total number of conversations in this folder that are in an active '
        'state (vs pending).',
    )
    modified_at = properties.DateTime(
        'UTC time when this folder was modified.',
    )



# -*- coding: utf-8 -*-
# Copyright 2017-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import properties

from .folder import Folder
from .mailbox_ref import MailboxRef


class Mailbox(MailboxRef):

    slug = properties.String(
        'Key used to represent this Mailbox.',
        required=True,
    )
    email = properties.String(
        'Email address',
        required=True,
    )
    folders = properties.List(
        'Folders that this mailbox contains.',
        prop=Folder,
    )
    created_at = properties.DateTime(
        'UTC time when this mailbox was created.',
    )
    modified_at = properties.DateTime(
        'UTC time when this mailbox was modified.',
    )



# -*- coding: utf-8 -*-
# Copyright 2017-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import properties

from .. import BaseModel


class Field(BaseModel):
    """The field object represents a user's response to a custom field
       in the context of a single conversation."""

    field_id = properties.Integer(
        'The field definition identifier; note that multiple conversations '
        'will each have values for the same field identifier.',
    )
    name = properties.String(
        'The name of the field; note that this may change if a field is '
        'renamed, but the ``field_id`` will not.',
        required=True,
    )
    value = properties.String(
        'The value the user specified for the field.',
    )
    type = properties.StringChoice(
        'Type of the custom field.',
        choices=['SINGLE_LINE',
                 'MULTI_LINE',
                 'DATE',
                 'NUMBER',
                 'DROPDOWN',
                 ],
    )
    label = properties.String(
        'String representation of the custom field\'s value. Unlike '
        '``value`` it contains the actual dropdown option value for '
        '``DROPDOWN`` custom field.',
    )



# -*- coding: utf-8 -*-
# Copyright 2017-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import properties

from .. import BaseModel

from .attachment import Attachment
from .email import Email
from .mailbox_ref import MailboxRef
from .person import Person
from .source import Source


class Thread(BaseModel):

    assigned_to = properties.Instance(
        'The Help Scout user assigned to this conversation.',
        instance_class=Person,
    )
    status = properties.StringChoice(
        'Status of the thread.',
        choices=['nochange', 'active', 'pending', 'closed', 'spam'],
    )
    created_at = properties.DateTime(
        'UTC time when this thread was created.',
    )
    opened_at = properties.DateTime(
        'UTC time when this thread was viewed by the customer. Only applies '
        'to threads with a ``type`` of message.',
    )
    created_by = properties.Instance(
        'The Person who created this thread. The ``type`` property will '
        'specify whether it was created by a ``user`` or a ``customer``.',
        instance_class=Person,
    )
    source = properties.Instance(
        'Origin of thread',
        instance_class=Source,
    )
    action_type = properties.StringChoice(
        'Describes an optional action associated with the line item.',
        choices=['movedFromMailbox',
                 'merged',
                 'imported',
                 'workflow',
                 'importedExternal',
                 'changedTicketCustomer',
                 'deletedTicket',
                 'restoreTicket',
                 ],
    )
    action_source_id = properties.Integer(
        'Associated with the ``action_type`` property. \n'
        '* If ``action_type`` is ``movedFromMailbox``, this will be the id '
        '  of the mailbox the conversation was moved from. \n'
        '* If ``action_type`` is ``merged``, this will be the id of the '
        '  original conversation. \n'
        '* If ``action_type`` is ``imported``, this will be null. \n'
        '* If ``action_type`` is ``workflow``, this will be the id of the '
        '  workflow that was run.',
    )
    from_mailbox = properties.Instance(
        'If the conversation was moved, this represents the MailboxRef from '
        'which it was moved.',
        instance_class=MailboxRef,
    )
    type = properties.StringChoice(
        'The type of thread. \n'
        'A ``lineitem`` represents a change of state on the conversation. '
        'This could include, but not limited to, the conversation was '
        'assigned, the status changed, the conversation was moved from one '
        'mailbox to another, etc. A line item won\'t have a body, to/cc/bcc '
        'lists, or attachments. \n'
        'When a conversation is forwarded, a new conversation is created to '
        'represent the forwarded conversation. \n'
        '``forwardparent ``is the type set on the thread of the original '
        'conversation that initiated the forward event. \n'
        '``forwardchild`` is the type set on the first thread of the new '
        'forwarded conversation.',
        choices=['lineitem',
                 'note',
                 'message',
                 'chat',
                 'customer',
                 'forwardparent',
                 'forwardchild',
                 'phone',
                 ],
    )
    state = properties.StringChoice(
        'The state of the thread. \n'
        'A state of ``underreview`` means the thread has been stopped by '
        'Traffic Cop and is waiting to be confirmed (or discarded) by the '
        'person that created the thread. \n'
        'A state of ``hidden`` means the thread was hidden (or removed) from '
        'customer-facing emails.',
        choices=['pubished', 'draft', 'underreview', 'hidden'],
    )
    customer = properties.Instance(
        'If thread type is ``message``, this is the customer associated with '
        'the conversation. \n'
        'If thread type is ``customer``, this is the the customer who '
        'initiated the thread.',
        instance_class=Person,
    )
    body = properties.String(
        'Body text',
    )
    to = properties.List(
        'Email to',
        prop=Email,
    )
    cc = properties.List(
        'CC to',
        prop=Email,
    )
    bcc = properties.List(
        'BCC to',
        prop=Email,
    )
    attachments = properties.List(
        'Attachments',
        prop=Attachment,
    )
    saved_reply_id = properties.Integer(
        'ID of Saved reply that was used to create this Thread.',
    )
    created_by_customer = properties.Bool(
        'Equivalent to ``created_by.type == "customer"``.',
    )



# -*- coding: utf-8 -*-
# Copyright 2017-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import unittest

from ..exceptions import HelpScoutRemoteException, HelpScoutValidationError

class TestExceptions(unittest.TestCase):

    def test_helpscout_remote_exception(self):
        """It should have the proper str representation."""
        code = 404
        message = 'A Fail'
        e = HelpScoutRemoteException(code, message)
        self.assertEqual(str(e), '(404) A Fail')

    def test_helpscout_base_exception(self):
        """It should include the message in string."""
        message = 'message'
        e = HelpScoutValidationError(message)
        self.assertEqual(str(e), message)



# -*- coding: utf-8 -*-
# Copyright 2016-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

from setuptools import Command, setup
from setuptools import find_packages
from unittest import TestLoader, TextTestRunner

from os import environ, path


PROJECT = 'helpscout'
SHORT_DESC = 'This library allows you to interact with HelpScout using ' \
             'Python.'
README_FILE = 'README.rst'

CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Environment :: Console',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python',
    'Topic :: Software Development :: Libraries :: Python Modules',
]

version = environ.get('RELEASE') or environ.get('VERSION') or '0.0.0'

if environ.get('TRAVIS_BUILD_NUMBER'):
    version += 'b%s' % environ.get('TRAVIS_BUILD_NUMBER')


setup_vals = {
    'name': PROJECT,
    'author': 'LasLabs Inc.',
    'author_email': 'support@laslabs.com',
    'description': SHORT_DESC,
    'url': 'https://laslabs.github.io/python-%s' % PROJECT,
    'download_url': 'https://github.com/LasLabs/python-%s' % PROJECT,
    'license': 'MIT',
    'classifiers': CLASSIFIERS,
    'version': version,
}


if path.exists(README_FILE):
    with open(README_FILE) as fh:
        setup_vals['long_description'] = fh.read()


install_requires = []
if path.exists('requirements.txt'):
    with open('requirements.txt') as fh:
        install_requires = fh.read().splitlines()


class FailTestException(Exception):
    """ It provides a failing build """
    pass


class Tests(Command):
    """ Run test & coverage, save reports as XML """

    user_options = []  # < For Command API compatibility

    def initialize_options(self, ):
        pass

    def finalize_options(self, ):
        pass

    def run(self, ):
        loader = TestLoader()
        tests = loader.discover('.', 'test_*.py')
        t = TextTestRunner(verbosity=1)
        res = t.run(tests)
        if not res.wasSuccessful():
            raise FailTestException()


if __name__ == "__main__":
    setup(
        packages=find_packages(exclude=('tests')),
        cmdclass={'test': Tests},
        tests_require=[
            'mock',
        ],
        install_requires=install_requires,
        **setup_vals
    )



# -*- coding: utf-8 -*-
# Copyright 2017-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import unittest

from contextlib import contextmanager

try:
    import mock
except ImportError:
    mock = unittest.mock

from ..request_paginator import RequestPaginator


class TestRequestPaginator(unittest.TestCase):

    def setUp(self):
        self.vals = {
            'endpoint': 'endpoint',
            'data': {'param': 1},
            'output_type': dict,
        }
        self.test_responses = [
            {
                'page': 1,
                'pages': 3,
                'items': [{
                    'page': 1,
                }],
            },
            {
                'page': 2,
                'pages': 3,
                'items': [{
                    'page': 2,
                }],
            },
            {
                'page': 3,
                'pages': 3,
                # This one emulates a singleton
                'item': {
                    'page': 3,
                },
            },
        ]
        self.paginator = RequestPaginator(**self.vals)

    @contextmanager
    def mock_session(self, response_code=200, responses=None):
        if responses is None:
            responses = self.test_responses
        with mock.patch.object(self.paginator, 'session') as session:
            response = mock.MagicMock()
            response.status_code = response_code
            response.json.side_effect = responses
            session.get.return_value = response
            session.post.return_value = response
            yield session

    def test_init_attrs(self):
        """ It should correctly assign instance attributes. """
        attrs = {
            attr: getattr(self.paginator, attr) for attr in self.vals.keys()
        }
        self.assertDictEqual(attrs, self.vals)

    @mock.patch('helpscout.request_paginator.requests')
    def test_init_session(self, requests):
        """ It should initialize a requests session. """
        paginator = RequestPaginator(**self.vals)
        self.assertEqual(paginator.session, requests.Session())

    def test_get_gets(self):
        """ It should call the session with proper args. """
        params = {'param_test': 23234}
        with self.mock_session() as session:
            self.paginator.get(params)
            session.get.assert_called_once_with(
                url=self.vals['endpoint'],
                params=params,
                verify=True,
            )

    def test_get_return(self):
        """ It should return the json decoded response. """
        with self.mock_session() as _:
            res = self.paginator.get({})
            self.assertEqual(len(res), 1)
            self.assertDictEqual(res[0], self.test_responses[0]['items'][0])

    def test_post_posts(self):
        """ It should call the session with proper args. """
        params = {'param_test': 23234}
        with self.mock_session() as session:
            self.paginator.post(params)
            session.post.assert_called_once_with(
                url=self.vals['endpoint'],
                json=params,
                verify=True,
            )

    def test_post_return(self):
        """ It should return the json decoded response. """
        with self.mock_session() as _:
            res = self.paginator.post({})
            self.assertEqual(len(res), 1)
            self.assertDictEqual(res[0], self.test_responses[0]['items'][0])

    def test_call_get(self):
        """It should get when the request type is GET."""
        params = {'param_test': 23234}
        self.paginator.request_type = self.paginator.GET
        with mock.patch.object(self.paginator, 'get') as get:
            self.paginator.call(params)
        get.assert_called_once_with(params)

    def test_call_post(self):
        """It should post when the request type is POST."""
        params = {'param_test': 23234}
        self.paginator.request_type = self.paginator.POST
        with mock.patch.object(self.paginator, 'post') as post:
            self.paginator.call(params)
        post.assert_called_once_with(params)

    def test_iter(self):
        """ It should iterate until the end & yield data. """
        with self.mock_session() as session:
            session.get().json.side_effect = self.test_responses
            res = list(self.paginator)
            expect = [{'page': 1}, {'page': 2}, {'page': 3}]
            self.assertEqual(res, expect)



# -*- coding: utf-8 -*-
# Copyright 2017-TODAY LasLabs Inc.
# License MIT (https://opensource.org/licenses/MIT).

import properties

from .person import Person

from .address import Address
from .chat import Chat
from .email import Email
from .phone import Phone
from .social_profile import SocialProfile
from .website import Website


class Customer(Person):
    """This represents a customer, which is a type of person."""

    full_name = properties.String(
        'Full name for the customer',
    )
    photo_type = properties.StringChoice(
        'Type of photo.',
        choices=['unknown',
                 'gravatar',
                 'twitter',
                 'facebook',
                 'googleprofile',
                 'googleplus',
                 'linkedin',
                 ],
    )
    gender = properties.StringChoice(
        'Gender of this customer.',
        choices=['female', 'male', 'unknown'],
        default='unknown',
        required=True,
    )
    age = properties.String(
        'Age (or age range) of this customer.',
    )
    organization = properties.String(
        'Company/Organization the customer identifies with.',
    )
    job_title = properties.String(
        'Job title at company/organization.',
    )
    location = properties.String(
        'Location',
    )
    background = properties.String(
        'This is the Notes field from the user interface.',
    )
    address = properties.Instance(
        'The address for this customer.',
        instance_class=Address,
    )
    social_profiles = properties.List(
        'Social profiles that represent this customer.',
        prop=SocialProfile,
    )
    emails = properties.List(
        'Email addresses for this customer.',
        prop=Email,
    )
    phones = properties.List(
        'Phone numbers for this customer.',
        prop=Phone,
    )
    chats = properties.List(
        'Chat addresses for this customer.',
        prop=Chat,
    )
    websites = properties.List(
        'Websites for this customer.',
        prop=Website,
    )
    created_at = properties.DateTime(
        'UTC time when this customer was created.',
    )
    modified_at = properties.DateTime(
        'UTC time when this customer was modified.',
    )
